/// @file parsewav.h
///
/// @brief wavef[^ <-> gf[^ <-> VAf[^@ϊNX
///
/// @author Sasaji
/// @date   2011.7.1
///

#ifndef _PARSEWAV_H_
#define _PARSEWAV_H_

#include "../../common.h"
#include "../../fileio.h"
#include "parsewav_param.h"
#include "parsewav_buffer.h"
#include "parsewav_format.h"
#include "parsewav_util.h"
#include "parsewav_dft.h"

typedef struct pos_st {
	int    pos;
	uint32_t ms;
} pos_t;

class FILEIO;

/// @brief f[^R[_̕ϒg(FSK)͂ăVAf[^ɕϊNX
///
/// wavet@C`ŕۑꂽf[^R[_̕ϒg(FSK)1200Hz2400Hz̔g
/// ͂Ĕgf[^ƂɃVAf[^oB
/// tɃVAf[^ϒgwavet@C쐬B
class PARSEWAV
{
private:
	FILEIO *fio;
	FILEIO *flog;
	PARSEWAV_PARAM param;
	PARSEWAV_UTIL conv;
	PARSEWAV_UTIL snd;
	PARSEWAV_DFT  dft;

	int infile_type;

	// input file info
	size_t sample_start;
	size_t sample_pos;
	size_t sample_num;
	size_t sample_num_orig;
	uint32_t sample_ms;

	int	phase1;
	int phase2;
	int start_phase;

	int phase2_frip;

	int baud24_frip;

//	int half_wave;

	struct parse_carrier_st {
		double x0_prev;
		int    wav_prev;
		int    sample_cnt;
		int    odd;
		int    carr_prev;
	} st_pa_carr;

	struct lamda_st {
		double samples[3];	// 1g̃Tv 0:1200Hz 1:2400Hz 2:4800Hz
		double us_delta;	// 1Tv̒(us)
		double us_range[3];	// 0:1200Hz 1:2400Hz 2:4800Hz
		double us[3];		// 0:1200Hz 1:2400Hz 2:4800Hz
		double us_min[3];
		double us_max[3];
		double us_avg[3];
		double us_mid[2];	// 0:1800Hz 1:3600Hz
		double us_mid_avg[2];
		double us_limit[2];	// 0:9600Hz 1:19200Hz
	} st_lamda;

	Buf_t<uint8_t> *wave_data;
	Buf_t<uint8_t> *wave_correct_data;
	Buf_t<char> *carrier_data;

	int err_num;
	int rep_p1_sample_num[9];

	PARSEWAV_FORMAT inwav;
	PARSEWAV_FORMAT outwav;
	PARSEWAV_FORMAT sndwav;

	int infile_samples_onebit;
	int infile_samples_const;
	int inwav_dirpos;

//	double h[2][50];

	int check_l3c_format(FILEIO *fp);

//	int get_wav_sample(int);
	int get_wav_sample(int blk_size, int reverse);
	int skip_wav_sample(int dir);
	size_t get_wav_sample(Buf_t<uint8_t> *w_data, int size, int reverse);
	int skip_l3c_sample(int dir);
	size_t get_l3c_sample(Buf_t<char> *c_data, int size);

	void  init_parameters();

	int   decode_to_carrier_wave_bn(int fsk_spd, Buf_t<uint8_t> *w_data, Buf_t<char> *c_data);

	int   find_start_carrier_bit(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata, int *c_onelen);
	int   decode_to_serial_data(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata, int *c_onelen);
	bool  decode_phase1(int fsk_spd, int start_phase, int corret_type, int reverse, char *w_onedata, int *w_onelen);
	int   decode_phase2(char *s_data, char *c_onedata, int *c_onelen);

	int   encode_phase2(uint8_t data, uint8_t *buffer, int *c_len);
	int   encode_phase1(uint8_t data, uint8_t *buffer, int *c_len);

	int   get_sample_rate(int);

public:
	// prototype define
	PARSEWAV();
	~PARSEWAV();

	int   CheckFileFormat(FILEIO *fp, int file_type, int def_rate_num, int def_bits_num, int def_ch);

	int   InitData(FILEIO *fp, int file_type, int baud_rate, int w_onelen);
	void  ClearData();
	int   GetData(int dir, char *s_data, int &s_datalen, char *w_onedata, int *w_onelen, char *c_onedata, int *c_onelen);
	int   SetData(uint8_t data, int &data_len, uint8_t *buffer, int *c_len);
	int   InitFileHeader(uint8_t *buffer);
	void  SetFileHeader(FILEIO *fp);

	_TCHAR* Errmsg();

	void SetSampleRate(int value) { param.SetSampleRate(value); }
	void SetSampleBits(int value) {	param.SetSampleBits(value); }
	void SetBaud(int value)		{ param.SetBaud(value); }
	void SetReverseWave(int value) { param.SetReverseWave(value); }
	void SetHalfWave(int value) { param.SetHalfWave(value); }
	void SetCorrectWave(int value) { param.SetCorrectWave(value); }
	void SetCorrectType(int value) { param.SetCorrectType(value); }
	void SetFreq(int num, int value) { param.SetFreq(num, value); }

	void SetBaudAndFrequency(int baud_rate, int spd);
	void SetParseParamerers(int half, int reverse, int correct, int corr_type);
	void SetStartPhase(int type);

	int GetSampleRate(void) { return param.GetSampleRate(); }
	int GetSampleBits(void) { return param.GetSampleBits(); }
	int GetBaud(void)		{ return param.GetBaud(); }
	int GetReverseWave(void) { return param.GetReverseWave(); }
	int GetHalfWave(void)	{ return param.GetHalfWave(); }
	int GetCorrectWave(void) { return param.GetCorrectWave(); }
	int GetCorrectType(void) { return param.GetCorrectType(); }
	int GetFreq(int num) { return param.GetFreq(num); }
//	int GetOneWave(void) { return (int)st_lamda.samples[0]; }
	size_t GetSamplePos(void) { return sample_pos; }

	void SetFormatForSound(int rate, int bits, int ch);
};

#endif
