﻿/// @file parsewav_dft.cpp
///
/// @author Sasaji
/// @date   2017.12.01
///

#include <string.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "parsewav_dft.h"

/// @brief コンストラクタ
///
PARSEWAV_DFT::PARSEWAV_DFT()
{
	samples = 0;
	memset(h, 0, sizeof(h));
}

/// @brief デストラクタ
///
PARSEWAV_DFT::~PARSEWAV_DFT()
{
}

/// @brief 補正用dftの係数を作成
/// @param[in] samples 1200Hzのサンプル数
/// @param[in] type 0:cos 1:sin
///
void PARSEWAV_DFT::Init(double samples, int type)
{
	int half = (int)(samples / 2.0);
	int n, k;

	for(k = 0; k < 2; k++) {
		for(n = 0; n < (int)samples; n++) {
			double rag = -2.0 * M_PI * (k + 1) * (n - half) / samples;
			if (type == 0) {
				h[k][n] = cos(rag);
			} else {
				h[k][n] = sin(rag);
			}
		}
	}

	this->samples = samples;
	a_max = 0;
	a_min = 0;

	return;
}

/// @brief フーリエ変換
/// 1200Hz, 2400Hz,4800Hzの正弦波or余弦波を合成して補正する
///
/// @param[in] w_data    元の波形データ
/// @param[out] wc_data  補正後の波形データ
/// @return 変換したデータ数
int PARSEWAV_DFT::Calcrate(Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data)
{
	int k;
	int n;
	int p;
	int wc;
	double dat;
	Buf_t<int> *f_data;

	int half = (int)(samples / 2.0);

	for (k = 0; k < 2; k++) {
		f_data = &freq_data[k];
		f_data->init();
	}
	for (p = half; p < (w_data->w_pos - half); p++) {
		for (k = 0; k < 2; k++) {
			f_data = &freq_data[k];

			dat = 0.0;
			for (n = p - half; n < (p + half); n++) {
				dat += (double)((int)w_data->data[n] - 128) * h[k][n - p + half];
			}
			int idat = (int)(dat * 1000.0 / samples);
			f_data->add(idat, 0);
		}
		wc = (freq_data[0].data[p - half] + freq_data[1].data[p - half]) / 1000;
		if (a_max < wc) a_max = wc;
		if (a_min > wc) a_min = wc;
		if (wc >= 128) wc = 127;
		if (wc < -128) wc = -128;
		wc_data->add((uint8_t)(wc + 128), w_data->get_ms(0));
		w_data->r_pos++;
		w_data->total_r_pos++;
		if (wc_data->isFull(0)) {
			break;
		}
	}
#if 0
	for (p = 0; p < (wc_data->w_pos); p++) {
//		if (param.debug_log) {
		sprintf(cbuff,"%d,%d,%d,%d,%d,%d\n"
			, (w_data->data[p + half] - 128) * 1000
			, freq_data[0].re[p]
			, freq_data[1].re[p]
			, freq_data[2].re[p]
			, (w_data->data[p + half] - 128)
			, (wc_data->data[p + wc_data->start_pos] - 128)
		);
		if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
//		}
	}
#endif
	return wc_data->w_pos;
}
